@@ -8,13 +8,13 @@ from attr.validators import instance_of
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
-from typing import List, Iterator, Tuple, Iterable, Dict
+from typing import List, Iterator, Tuple, Iterable, Dict, Generator
 from numba import jit, float64, uint64, int64
 from collections import OrderedDict, namedtuple, deque
 import warnings
 import h5py_cache
-from progress.bar import ChargingBar
 from tqdm import tqdm
+from pysight.rectify_lines import LineRectifier
 
 
 def trunc_end_of_file(name) -> str:
@@ -32,6 +32,7 @@ class Movie(object):
     A holder for Volume objects to be displayed consecutively.
     """
     data            = attr.ib()
+    lines           = attr.ib(validator=instance_of(pd.Series))
     reprate         = attr.ib(default=80e6, validator=instance_of(float))
     x_pixels        = attr.ib(default=512, validator=instance_of(int))
     y_pixels        = attr.ib(default=512, validator=instance_of(int))
@@ -39,7 +40,7 @@ class Movie(object):
     name            = attr.ib(default='Movie', validator=instance_of(str),
                               convert=trunc_end_of_file)
     binwidth        = attr.ib(default=800e-12, validator=instance_of(float))
-    fill_frac       = attr.ib(default=80.0, validator=instance_of(float))
+    fill_frac       = attr.ib(default=71.0, validator=instance_of(float))
     big_tiff        = attr.ib(default=True, validator=instance_of(bool))
     bidir           = attr.ib(default=True, validator=instance_of(bool))
     num_of_channels = attr.ib(default=1, validator=instance_of(int))
@@ -50,6 +51,10 @@ class Movie(object):
     exp_params      = attr.ib(default={}, validator=instance_of(dict))
     line_delta      = attr.ib(default=158000, validator=instance_of(int))
     use_sweeps      = attr.ib(default=False, validator=instance_of(bool))
+    cache_size      = attr.ib(default=10*1024**3, validator=instance_of(int))
+    tag_as_phase    = attr.ib(default=True, validator=instance_of(bool))
+    tag_freq        = attr.ib(default=189e3, validator=instance_of(float))
+    mirror_phase    = attr.ib(default=-2.71, validator=instance_of(float))
     summed_mem      = attr.ib(init=False)
     stack           = attr.ib(init=False)
     summed_to_file  = attr.ib(init=False)
@@ -85,12 +90,12 @@ class Movie(object):
         photons_per_pulse = {}
         if self.num_of_channels == 1:
             photons_per_pulse[1] = self.data.shape[0] / num_of_pulses
-            return photons_per_pulse
         else:
             for chan in range(self.num_of_channels):
                 photons_per_pulse[chan] = self.data.loc[chan]
+        return photons_per_pulse
 
-    def run(self):
+    def run(self) -> None:
         """
         Main pipeline for the movie object
         :return:
@@ -99,7 +104,7 @@ class Movie(object):
         self.__print_outputs()
         print("Movie object created, analysis done.")
 
-    def gen_of_volumes(self, channel_num: int) -> Iterator:
+    def gen_of_volumes(self, channel_num: int) -> Generator:
         """
         Populate the deque containing the volumes as a generator.
         Creates a list for each channel in the data. Channels start with 1.
@@ -108,20 +113,14 @@ class Movie(object):
         list_of_frames: List[int] = self.list_of_volume_times  # saves a bit of computation
         for idx, current_time in enumerate(list_of_frames[:-1]):  # populate deque with frames
             cur_data = self.data.xs(key=(current_time, channel_num), level=('Frames', 'Channel'), drop_level=False)
-            if not cur_data.empty:
-                yield Volume(data=cur_data, x_pixels=self.x_pixels, y_pixels=self.y_pixels,
-                             z_pixels=self.z_pixels, number=idx, abs_start_time=current_time,
-                             reprate=self.reprate, binwidth=self.binwidth, empty=False,
-                             end_time=(list_of_frames[idx + 1] - list_of_frames[idx]),
-                             bidir=self.bidir, fill_frac=self.fill_frac, censor=self.censor,
-                             line_delta=self.line_delta, use_sweeps=self.use_sweeps)
-            else:
-                yield Volume(data=cur_data, x_pixels=self.x_pixels,
-                             y_pixels=self.y_pixels, z_pixels=self.z_pixels, number=idx,
-                             reprate=self.reprate, binwidth=self.binwidth, empty=True,
-                             end_time=(list_of_frames[idx + 1] - list_of_frames[idx]),
-                             bidir=self.bidir, fill_frac=self.fill_frac, censor=self.censor,
-                             line_delta=self.line_delta, use_sweeps=self.use_sweeps)
+            cur_lines = self.lines[(self.lines >= current_time) & (self.lines <list_of_frames[idx + 1])]
+            yield Volume(data=cur_data, lines=cur_lines, x_pixels=self.x_pixels, y_pixels=self.y_pixels,
+                         z_pixels=self.z_pixels, number=idx, abs_start_time=current_time,
+                         reprate=self.reprate, binwidth=self.binwidth, empty=True if cur_data.empty else False,
+                         end_time=(list_of_frames[idx + 1] - list_of_frames[idx]),
+                         bidir=self.bidir, fill_frac=self.fill_frac, censor=self.censor,
+                         line_delta=self.line_delta, use_sweeps=self.use_sweeps,
+                         tag_as_phase=self.tag_as_phase, tag_freq=self.tag_freq, mirror_phase=self.mirror_phase)
 
     def __create_outputs(self) -> None:
         """
@@ -148,15 +147,16 @@ class Movie(object):
 
         else:
             if 'stack' in self.outputs:
-                self.outputs['stack'] = h5py_cache.File(f'{self.name[:-4]}.hdf5', 'a', chunk_cache_mem_size=100*1024**2,
+                self.outputs['stack'] = h5py_cache.File(f'{self.outputs["filename"]}', 'a',
+                                                        chunk_cache_mem_size=self.cache_size,
                                                         libver='latest', w0=1).require_group('Full Stack')
                 funcs_to_execute_during.append(self.__save_stack_incr)
                 funcs_to_execute_end.append(self.__close_file)
 
             if 'summed' in self.outputs:
-                self.summed_to_file = {i: 0 for i in range(1, self.num_of_channels + 1)}
+                self.summed_mem = {i: 0 for i in range(1, self.num_of_channels + 1)}
                 funcs_to_execute_during.append(self.__append_summed_data)
-                funcs_to_execute_end.append(self.__save_summed_file)
+                funcs_to_execute_end.append(self.__save_summed_at_once)
 
         VolTuple = namedtuple('VolumeHist', ('hist', 'edges'))
         data_of_vol = VolTuple
@@ -176,33 +176,33 @@ class Movie(object):
         for func in funcs_to_execute_end:
             func()
 
-    def __save_stack_at_once(self):
+    def __save_stack_at_once(self) -> None:
         """ Save the entire in-memory stack into .hdf5 file """
-        with h5py_cache.File(f'{self.name[:-4]}.hdf5', 'a', chunk_cache_mem_size=100*1024**2,
+        with h5py_cache.File(f'{self.outputs["filename"]}', 'a', chunk_cache_mem_size=self.cache_size,
                              libver='latest', w0=1) as f:
-            print("Saving full stack to disk.")
+            print("Saving full stack to disk...")
             for channel in range(1, self.num_of_channels + 1):
                 f["Full Stack"][f"Channel {channel}"][...] = self.stack[channel]
 
-    def __save_summed_at_once(self):
+    def __save_summed_at_once(self) -> None:
         """ Save the netire in-memory summed data into .hdf5 file """
-        with h5py_cache.File(f'{self.name[:-4]}.hdf5', 'a', chunk_cache_mem_size=100*1024**2,
+        with h5py_cache.File(f'{self.outputs["filename"]}', 'a', chunk_cache_mem_size=self.cache_size,
                              libver='latest', w0=1) as f:
             for channel in range(1, self.num_of_channels + 1):
-                f["Summed Stack"][f"Channel {channel}"][...] = self.summed_mem[channel]
+                f["Summed Stack"][f"Channel {channel}"][...] = np.squeeze(self.summed_mem[channel])
 
-    def __close_file(self):
+    def __close_file(self) -> None:
         """ Close the file pointer of the specific channel """
         self.outputs['stack'].file.close()
 
-    def __convert_deque_to_arr(self):
+    def __convert_deque_to_arr(self) -> None:
         """ Convert a deque with a bunch of frames into a single numpy array with an extra
         dimension (0) containing the data.
         """
         for channel in range(1, self.num_of_channels + 1):
-            self.stack[channel] = np.stack(self.stack[channel], axis=-1)
+            self.stack[channel] = np.squeeze(np.stack(self.stack[channel], axis=0))
 
-    def __create_memory_output(self, data: np.ndarray, channel: int, **kwargs):
+    def __create_memory_output(self, data: np.ndarray, channel: int, **kwargs) -> None:
         """
         If the user desired, create two memory constructs -
         A summed array of all images (for a specific channel), and a stack containing
@@ -213,33 +213,22 @@ class Movie(object):
         self.stack[channel].append(data)
         self.summed_mem[channel] += np.uint16(data)
 
-    def __save_stack_incr(self, data: np.ndarray, channel: int, vol_num: int):
+    def __save_stack_incr(self, data: np.ndarray, channel: int, vol_num: int) -> None:
         """
         Save incrementally new data to an open file on the disk
         :param data: Data to save
         :param channel: Current spectral channel of data
         :param vol_num: Current volume
         """
-        self.outputs['stack'][f'Channel {channel}'][...,vol_num] = data
+        self.outputs['stack'][f'Channel {channel}'][vol_num, ...] = np.squeeze(data)
 
-    def __append_summed_data(self, data: np.ndarray, channel: int, **kwargs):
+    def __append_summed_data(self, data: np.ndarray, channel: int, **kwargs) -> None:
         """
         Create a summed variable later to be saved as the channel's data
         :param data: Data to be saved
         :param channel: Spectral channel of data to be saved
         """
-        self.summed_to_file[channel] += np.uint16(data)
-
-    def __save_summed_file(self):
-        """
-        Save once
-        :param channel:
-        :return:
-        """
-        with h5py_cache.File(f'{self.name[:-4]}.hdf5', 'a', chunk_cache_mem_size=100*1024**2,
-                             libver='latest', w0=1) as f:
-            for channel in range(1, self.num_of_channels + 1):
-                f['Summed Stack'][f'Channel {channel}'][...] = self.summed_to_file[channel]
+        self.summed_mem[channel] += np.uint16(data)
 
     def __print_outputs(self) -> None:
         """
@@ -250,7 +239,7 @@ class Movie(object):
 
         print('======================================================= \nOutputs:\n--------')
         if 'stack' in self.outputs:
-            print(f'Stack file created with name "{self.name[:-4]}.hdf5", \ncontaining a data group named'
+            print(f'Stack file created with name "{self.outputs["filename"]}", \ncontaining a data group named'
                   ' "Full Stack", with one dataset per channel.')
 
         if 'memory' in self.outputs:
@@ -258,13 +247,13 @@ class Movie(object):
                   'and in stacked form under `movie.summed_mem`.')
 
         if 'summed' in self.outputs:
-            print(f'Summed stack file created with name "{self.name[:-4]}.hdf5", \ncontaining a data group named'
+            print(f'Summed stack file created with name "{self.outputs["filename"]}", \ncontaining a data group named'
                   ' "Summed Stack", with one dataset per channel.')
 
     def __nano_flim(self, data: np.ndarray) -> None:
         pass
 
-    def show_summed(self, channel: int) -> None:
+    def show_summed(self, channel: int=1) -> None:
         """ Show the summed Movie """
 
         plt.figure()
@@ -281,14 +270,14 @@ class Movie(object):
         plt.title(f'Channel number {channel}')
         plt.axis('off')
 
-    def show_stack(self, channel: int, slice_range: Iterable) -> None:
+    def show_stack(self, channel: int=1, slice_range: Iterable=range(10)) -> None:
         """ Show the stack of given slices """
         if 'time_rel_pulse' in self.data.columns:
             self.__show_stack_flim(channel=channel, slice_range=slice_range)
         else:
             self.__show_stack_no_flim(channel=channel, slice_range=slice_range)
 
-    def __show_stack_no_flim(self, channel: int, slice_range: Iterable):
+    def __show_stack_no_flim(self, channel: int, slice_range: Iterable) -> None:
         """ Show the slices from the generated stack """
 
         img = None
@@ -301,7 +290,7 @@ class Movie(object):
             plt.pause(0.1)
             plt.draw()
 
-    def __show_stack_flim(self, channel: int, slice_range: Iterable):
+    def __show_stack_flim(self, channel: int, slice_range: Iterable) -> None:
         """ Show the slices from the generated stack that contains FLIM data """
 
         for frame in slice_range:
@@ -312,9 +301,10 @@ class Movie(object):
 @attr.s(slots=True)
 class Volume(object):
     """
-    A Movie() is a sequence of volumes. Each volume contains frames in a plane.
+    A Movie() is a sequence of Volumes(). Each volume contains frames in a plane.
     """
     data           = attr.ib(validator=instance_of(pd.DataFrame))
+    lines          = attr.ib(validator=instance_of(pd.Series))
     x_pixels       = attr.ib(default=512, validator=instance_of(int))
     y_pixels       = attr.ib(default=512, validator=instance_of(int))
     z_pixels       = attr.ib(default=1, validator=instance_of(int))
@@ -329,6 +319,9 @@ class Volume(object):
     censor         = attr.ib(default=False, validator=instance_of(bool))
     line_delta     = attr.ib(default=158000, validator=instance_of(int))
     use_sweeps     = attr.ib(default=False, validator=instance_of(bool))
+    tag_as_phase   = attr.ib(default=True, validator=instance_of(bool))
+    tag_freq       = attr.ib(default=189e3, validator=instance_of(float))
+    mirror_phase   = attr.ib(default=-2.76, validator=instance_of(float))  # phase for scanning mirrors
 
     @property
     def metadata(self) -> OrderedDict:
@@ -341,11 +334,11 @@ class Volume(object):
         metadata = OrderedDict()
         jitter = 0.02  # 2% of jitter of the signals that creates volumes
 
-        # Volume metadata
+        # Volume metadata - rows
         volume_start: int = 0
         metadata['Volume'] = Struct(start=volume_start, end=self.end_time, num=self.x_pixels+1)
 
-        # y-axis metadata
+        # y-axis metadata - columns
         y_start, y_end = metadata_ydata(data=self.data, jitter=jitter, bidir=self.bidir,
                                         fill_frac=self.fill_frac, delta=self.line_delta,
                                         sweeps=self.use_sweeps)
@@ -356,8 +349,8 @@ class Volume(object):
 
         # z-axis metadata
         if 'Phase' in self.data.columns:
-            z_start = -1
-            z_end = 1
+            z_start = -1 if self.tag_as_phase else 0
+            z_end = 1 if self.tag_as_phase else self.tag_period
             metadata['Z'] = Struct(start=z_start, end=z_end, num=self.z_pixels + 1)
 
         # Laser pulses metadata
@@ -374,7 +367,15 @@ class Volume(object):
 
         return metadata
 
-    def __create_hist_edges(self):
+    @property
+    def tag_period(self) -> int:
+        return int(np.ceil(1 / (self.tag_freq * self.binwidth)))
+
+    @property
+    def dimensions_iterable(self) -> List:
+        return [self.x_pixels, self.y_pixels, self.z_pixels, int(np.ceil(1 / (self.reprate * self.binwidth)))]
+
+    def __create_hist_edges(self) -> Tuple[List, int]:
         """
         Create three vectors that will create the grid of the frame. Uses Numba internal function for optimization.
         :return: Tuple of np.array
@@ -382,10 +383,19 @@ class Volume(object):
         metadata = self.metadata
         list_of_edges = []
 
-        if self.empty is not True:
+        if not self.empty:
             for num_of_dims, key in enumerate(metadata, 1):
                 if 'Volume' == key:
-                    list_of_edges.append(self.__create_line_array())
+                    try:
+                        list_of_edges.append(
+                            LineRectifier(lines=self.lines.values - self.abs_start_time,
+                                          x_pixels=self.x_pixels,
+                                          bidir=self.bidir,
+                                          end_time=self.end_time).rectify()
+                                            )
+                    except ValueError:  # problem with line correction\interpolation
+                        warnings.warn(f"\nVolume {self.number} contained too many missing\corrupt lines.")
+                        list_of_edges.append(np.arange(self.x_pixels + 1))
                 else:
                     list_of_edges.append(create_linspace(start=metadata[key].start,
                                                          stop=metadata[key].end,
@@ -393,24 +403,7 @@ class Volume(object):
 
             return list_of_edges, num_of_dims
         else:
-            return list(np.ones(len(metadata)))
-
-    def __create_line_array(self):
-        """
-        Generates the edges of the final histogram using the line signal from the data
-        :return: np.array
-        """
-        lines = self.data.index.get_level_values('Lines').categories.values
-        lines.sort()
-        if len(lines) > 1:
-            if len(lines) < self.x_pixels:
-                raise ValueError(f'Not enough line events in volume number {self.number}.\n'
-                                 f'Only {len(lines)} were recorded.')
-            else:
-                mean_diff = np.diff(lines).mean()
-                return np.r_[lines[:self.x_pixels], np.array([lines[self.x_pixels - 1] + mean_diff], dtype='uint64')]
-        else:  # single pixel frames, perhaps
-            return np.r_[lines, lines + self.end_time]
+            return [], len(metadata)
 
     def create_hist(self) -> Tuple[np.ndarray, Iterable]:
         """
@@ -419,9 +412,8 @@ class Volume(object):
         """
 
         list_of_data_columns = []
-
+        list_of_edges, num_of_dims = self.__create_hist_edges()
         if not self.empty:
-            list_of_edges, num_of_dims = self.__create_hist_edges()
             list_of_data_columns.append(self.data['time_rel_frames'].values)
             list_of_data_columns.append(self.data['time_rel_line'].values)
             try:
@@ -439,13 +431,14 @@ class Volume(object):
             assert len(list_of_data_columns) == data_to_be_hist.shape[1]
 
             hist, edges = np.histogramdd(sample=data_to_be_hist, bins=list_of_edges)
-
+            if self.bidir:
+                hist[1::2] = np.fliplr(hist[1::2])
             if self.censor:
                 hist = self.__censor_correction(hist)
 
             return np.uint8(hist), edges
         else:
-            return np.zeros((self.x_pixels, self.y_pixels, self.z_pixels), dtype=np.uint8), (0, 0, 0)
+            return np.zeros(self.dimensions_iterable[:num_of_dims], dtype=np.uint8), (0, 0, 0)
 
     def __censor_correction(self, data) -> np.ndarray:
         """
@@ -476,9 +469,10 @@ def create_linspace(start, stop, num):
 
 
 def metadata_ydata(data: pd.DataFrame, jitter: float=0.02, bidir: bool=True, fill_frac: float=0,
-                   delta: int=158000, sweeps: bool=False):
+                   delta: int=158000, sweeps: bool=False) -> Tuple[int, int]:
     """
     Create the metadata for the y-axis.
+
     """
     lines_start: int = 0
 
